from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.process_dto_process_archive_transition_status import ProcessDtoProcessArchiveTransitionStatus
from ..models.process_dto_python_engine_status import ProcessDtoPythonEngineStatus
from ..models.process_dto_status import ProcessDtoStatus
from ..models.process_dto_type import ProcessDtoType
from qubicon.api.types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.offline_equipment_dto import OfflineEquipmentDto
    from ..models.online_equipment_dto import OnlineEquipmentDto
    from ..models.process_phase_dto import ProcessPhaseDto


T = TypeVar("T", bound="ProcessDto")


@_attrs_define
class ProcessDto:
    """
    Attributes:
        name (str):
        id (Union[Unset, int]):
        description (Union[Unset, str]):
        update_date (Union[Unset, int]):
        creation_date (Union[Unset, int]):
        gmp_enabled (Union[Unset, bool]):
        start_date (Union[Unset, int]):
        end_date (Union[Unset, int]):
        status (Union[Unset, ProcessDtoStatus]):
        publish_enabled (Union[Unset, bool]):
        publish_planned_at (Union[Unset, int]):
        imported (Union[Unset, bool]):
        import_date (Union[Unset, int]):
        last_usage_date (Union[Unset, int]):
        simulated (Union[Unset, bool]):
        python_engine_status (Union[Unset, ProcessDtoPythonEngineStatus]):
        type (Union[Unset, ProcessDtoType]):
        control_recipe_id (Union[Unset, int]):
        control_recipe_name (Union[Unset, str]):
        warm_up_date (Union[Unset, int]):
        python_online_calculator_restart_date (Union[Unset, int]):
        python_sampling_calculator_restart_date (Union[Unset, int]):
        media (Union[Unset, str]):
        buffer (Union[Unset, str]):
        process_archive_transition_status (Union[Unset, ProcessDtoProcessArchiveTransitionStatus]):
        current_phase (Union[Unset, ProcessPhaseDto]):
        online_equipment (Union[Unset, List['OnlineEquipmentDto']]):
        offline_equipment (Union[Unset, List['OfflineEquipmentDto']]):
        kpi_count (Union[Unset, int]):
        sensor_count (Union[Unset, int]):
        offline_sensor_count (Union[Unset, int]):
        online_sensor_count (Union[Unset, int]):
        has_draft_samplings (Union[Unset, bool]):
    """

    name: str
    id: Union[Unset, int] = UNSET
    description: Union[Unset, str] = UNSET
    update_date: Union[Unset, int] = UNSET
    creation_date: Union[Unset, int] = UNSET
    gmp_enabled: Union[Unset, bool] = UNSET
    start_date: Union[Unset, int] = UNSET
    end_date: Union[Unset, int] = UNSET
    status: Union[Unset, ProcessDtoStatus] = UNSET
    publish_enabled: Union[Unset, bool] = UNSET
    publish_planned_at: Union[Unset, int] = UNSET
    imported: Union[Unset, bool] = UNSET
    import_date: Union[Unset, int] = UNSET
    last_usage_date: Union[Unset, int] = UNSET
    simulated: Union[Unset, bool] = UNSET
    python_engine_status: Union[Unset, ProcessDtoPythonEngineStatus] = UNSET
    type: Union[Unset, ProcessDtoType] = UNSET
    control_recipe_id: Union[Unset, int] = UNSET
    control_recipe_name: Union[Unset, str] = UNSET
    warm_up_date: Union[Unset, int] = UNSET
    python_online_calculator_restart_date: Union[Unset, int] = UNSET
    python_sampling_calculator_restart_date: Union[Unset, int] = UNSET
    media: Union[Unset, str] = UNSET
    buffer: Union[Unset, str] = UNSET
    process_archive_transition_status: Union[Unset, ProcessDtoProcessArchiveTransitionStatus] = UNSET
    current_phase: Union[Unset, "ProcessPhaseDto"] = UNSET
    online_equipment: Union[Unset, List["OnlineEquipmentDto"]] = UNSET
    offline_equipment: Union[Unset, List["OfflineEquipmentDto"]] = UNSET
    kpi_count: Union[Unset, int] = UNSET
    sensor_count: Union[Unset, int] = UNSET
    offline_sensor_count: Union[Unset, int] = UNSET
    online_sensor_count: Union[Unset, int] = UNSET
    has_draft_samplings: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        id = self.id

        description = self.description

        update_date = self.update_date

        creation_date = self.creation_date

        gmp_enabled = self.gmp_enabled

        start_date = self.start_date

        end_date = self.end_date

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        publish_enabled = self.publish_enabled

        publish_planned_at = self.publish_planned_at

        imported = self.imported

        import_date = self.import_date

        last_usage_date = self.last_usage_date

        simulated = self.simulated

        python_engine_status: Union[Unset, str] = UNSET
        if not isinstance(self.python_engine_status, Unset):
            python_engine_status = self.python_engine_status.value

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        control_recipe_id = self.control_recipe_id

        control_recipe_name = self.control_recipe_name

        warm_up_date = self.warm_up_date

        python_online_calculator_restart_date = self.python_online_calculator_restart_date

        python_sampling_calculator_restart_date = self.python_sampling_calculator_restart_date

        media = self.media

        buffer = self.buffer

        process_archive_transition_status: Union[Unset, str] = UNSET
        if not isinstance(self.process_archive_transition_status, Unset):
            process_archive_transition_status = self.process_archive_transition_status.value

        current_phase: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.current_phase, Unset):
            current_phase = self.current_phase.to_dict()

        online_equipment: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.online_equipment, Unset):
            online_equipment = []
            for online_equipment_item_data in self.online_equipment:
                online_equipment_item = online_equipment_item_data.to_dict()
                online_equipment.append(online_equipment_item)

        offline_equipment: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.offline_equipment, Unset):
            offline_equipment = []
            for offline_equipment_item_data in self.offline_equipment:
                offline_equipment_item = offline_equipment_item_data.to_dict()
                offline_equipment.append(offline_equipment_item)

        kpi_count = self.kpi_count

        sensor_count = self.sensor_count

        offline_sensor_count = self.offline_sensor_count

        online_sensor_count = self.online_sensor_count

        has_draft_samplings = self.has_draft_samplings

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if description is not UNSET:
            field_dict["description"] = description
        if update_date is not UNSET:
            field_dict["updateDate"] = update_date
        if creation_date is not UNSET:
            field_dict["creationDate"] = creation_date
        if gmp_enabled is not UNSET:
            field_dict["gmpEnabled"] = gmp_enabled
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if status is not UNSET:
            field_dict["status"] = status
        if publish_enabled is not UNSET:
            field_dict["publishEnabled"] = publish_enabled
        if publish_planned_at is not UNSET:
            field_dict["publishPlannedAt"] = publish_planned_at
        if imported is not UNSET:
            field_dict["imported"] = imported
        if import_date is not UNSET:
            field_dict["importDate"] = import_date
        if last_usage_date is not UNSET:
            field_dict["lastUsageDate"] = last_usage_date
        if simulated is not UNSET:
            field_dict["simulated"] = simulated
        if python_engine_status is not UNSET:
            field_dict["pythonEngineStatus"] = python_engine_status
        if type is not UNSET:
            field_dict["type"] = type
        if control_recipe_id is not UNSET:
            field_dict["controlRecipeId"] = control_recipe_id
        if control_recipe_name is not UNSET:
            field_dict["controlRecipeName"] = control_recipe_name
        if warm_up_date is not UNSET:
            field_dict["warmUpDate"] = warm_up_date
        if python_online_calculator_restart_date is not UNSET:
            field_dict["pythonOnlineCalculatorRestartDate"] = python_online_calculator_restart_date
        if python_sampling_calculator_restart_date is not UNSET:
            field_dict["pythonSamplingCalculatorRestartDate"] = python_sampling_calculator_restart_date
        if media is not UNSET:
            field_dict["media"] = media
        if buffer is not UNSET:
            field_dict["buffer"] = buffer
        if process_archive_transition_status is not UNSET:
            field_dict["processArchiveTransitionStatus"] = process_archive_transition_status
        if current_phase is not UNSET:
            field_dict["currentPhase"] = current_phase
        if online_equipment is not UNSET:
            field_dict["onlineEquipment"] = online_equipment
        if offline_equipment is not UNSET:
            field_dict["offlineEquipment"] = offline_equipment
        if kpi_count is not UNSET:
            field_dict["kpiCount"] = kpi_count
        if sensor_count is not UNSET:
            field_dict["sensorCount"] = sensor_count
        if offline_sensor_count is not UNSET:
            field_dict["offlineSensorCount"] = offline_sensor_count
        if online_sensor_count is not UNSET:
            field_dict["onlineSensorCount"] = online_sensor_count
        if has_draft_samplings is not UNSET:
            field_dict["hasDraftSamplings"] = has_draft_samplings

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.offline_equipment_dto import OfflineEquipmentDto
        from ..models.online_equipment_dto import OnlineEquipmentDto
        from ..models.process_phase_dto import ProcessPhaseDto

        d = src_dict.copy()
        name = d.pop("name")

        id = d.pop("id", UNSET)

        description = d.pop("description", UNSET)

        update_date = d.pop("updateDate", UNSET)

        creation_date = d.pop("creationDate", UNSET)

        gmp_enabled = d.pop("gmpEnabled", UNSET)

        start_date = d.pop("startDate", UNSET)

        end_date = d.pop("endDate", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ProcessDtoStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = ProcessDtoStatus(_status)

        publish_enabled = d.pop("publishEnabled", UNSET)

        publish_planned_at = d.pop("publishPlannedAt", UNSET)

        imported = d.pop("imported", UNSET)

        import_date = d.pop("importDate", UNSET)

        last_usage_date = d.pop("lastUsageDate", UNSET)

        simulated = d.pop("simulated", UNSET)

        _python_engine_status = d.pop("pythonEngineStatus", UNSET)
        python_engine_status: Union[Unset, ProcessDtoPythonEngineStatus]
        if isinstance(_python_engine_status, Unset):
            python_engine_status = UNSET
        else:
            python_engine_status = ProcessDtoPythonEngineStatus(_python_engine_status)

        _type = d.pop("type", UNSET)
        type: Union[Unset, ProcessDtoType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = ProcessDtoType(_type)

        control_recipe_id = d.pop("controlRecipeId", UNSET)

        control_recipe_name = d.pop("controlRecipeName", UNSET)

        warm_up_date = d.pop("warmUpDate", UNSET)

        python_online_calculator_restart_date = d.pop("pythonOnlineCalculatorRestartDate", UNSET)

        python_sampling_calculator_restart_date = d.pop("pythonSamplingCalculatorRestartDate", UNSET)

        media = d.pop("media", UNSET)

        buffer = d.pop("buffer", UNSET)

        _process_archive_transition_status = d.pop("processArchiveTransitionStatus", UNSET)
        process_archive_transition_status: Union[Unset, ProcessDtoProcessArchiveTransitionStatus]
        if isinstance(_process_archive_transition_status, Unset):
            process_archive_transition_status = UNSET
        else:
            process_archive_transition_status = ProcessDtoProcessArchiveTransitionStatus(
                _process_archive_transition_status
            )

        _current_phase = d.pop("currentPhase", UNSET)
        current_phase: Union[Unset, ProcessPhaseDto]
        if isinstance(_current_phase, Unset):
            current_phase = UNSET
        else:
            current_phase = ProcessPhaseDto.from_dict(_current_phase)

        online_equipment = []
        _online_equipment = d.pop("onlineEquipment", UNSET)
        for online_equipment_item_data in _online_equipment or []:
            online_equipment_item = OnlineEquipmentDto.from_dict(online_equipment_item_data)

            online_equipment.append(online_equipment_item)

        offline_equipment = []
        _offline_equipment = d.pop("offlineEquipment", UNSET)
        for offline_equipment_item_data in _offline_equipment or []:
            offline_equipment_item = OfflineEquipmentDto.from_dict(offline_equipment_item_data)

            offline_equipment.append(offline_equipment_item)

        kpi_count = d.pop("kpiCount", UNSET)

        sensor_count = d.pop("sensorCount", UNSET)

        offline_sensor_count = d.pop("offlineSensorCount", UNSET)

        online_sensor_count = d.pop("onlineSensorCount", UNSET)

        has_draft_samplings = d.pop("hasDraftSamplings", UNSET)

        process_dto = cls(
            name=name,
            id=id,
            description=description,
            update_date=update_date,
            creation_date=creation_date,
            gmp_enabled=gmp_enabled,
            start_date=start_date,
            end_date=end_date,
            status=status,
            publish_enabled=publish_enabled,
            publish_planned_at=publish_planned_at,
            imported=imported,
            import_date=import_date,
            last_usage_date=last_usage_date,
            simulated=simulated,
            python_engine_status=python_engine_status,
            type=type,
            control_recipe_id=control_recipe_id,
            control_recipe_name=control_recipe_name,
            warm_up_date=warm_up_date,
            python_online_calculator_restart_date=python_online_calculator_restart_date,
            python_sampling_calculator_restart_date=python_sampling_calculator_restart_date,
            media=media,
            buffer=buffer,
            process_archive_transition_status=process_archive_transition_status,
            current_phase=current_phase,
            online_equipment=online_equipment,
            offline_equipment=offline_equipment,
            kpi_count=kpi_count,
            sensor_count=sensor_count,
            offline_sensor_count=offline_sensor_count,
            online_sensor_count=online_sensor_count,
            has_draft_samplings=has_draft_samplings,
        )

        process_dto.additional_properties = d
        return process_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.gmp_track_confirmation_dto_role import GmpTrackConfirmationDtoRole
from qubicon.api.types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user_dto import UserDto


T = TypeVar("T", bound="GmpTrackConfirmationDto")


@_attrs_define
class GmpTrackConfirmationDto:
    """
    Attributes:
        user (Union[Unset, UserDto]):
        when (Union[Unset, int]):
        approval (Union[Unset, bool]):
        confirmed_date (Union[Unset, int]):
        comment (Union[Unset, str]):
        role (Union[Unset, GmpTrackConfirmationDtoRole]):
    """

    user: Union[Unset, "UserDto"] = UNSET
    when: Union[Unset, int] = UNSET
    approval: Union[Unset, bool] = UNSET
    confirmed_date: Union[Unset, int] = UNSET
    comment: Union[Unset, str] = UNSET
    role: Union[Unset, GmpTrackConfirmationDtoRole] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user, Unset):
            user = self.user.to_dict()

        when = self.when

        approval = self.approval

        confirmed_date = self.confirmed_date

        comment = self.comment

        role: Union[Unset, str] = UNSET
        if not isinstance(self.role, Unset):
            role = self.role.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if user is not UNSET:
            field_dict["user"] = user
        if when is not UNSET:
            field_dict["when"] = when
        if approval is not UNSET:
            field_dict["approval"] = approval
        if confirmed_date is not UNSET:
            field_dict["confirmedDate"] = confirmed_date
        if comment is not UNSET:
            field_dict["comment"] = comment
        if role is not UNSET:
            field_dict["role"] = role

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_dto import UserDto

        d = src_dict.copy()
        _user = d.pop("user", UNSET)
        user: Union[Unset, UserDto]
        if isinstance(_user, Unset):
            user = UNSET
        else:
            user = UserDto.from_dict(_user)

        when = d.pop("when", UNSET)

        approval = d.pop("approval", UNSET)

        confirmed_date = d.pop("confirmedDate", UNSET)

        comment = d.pop("comment", UNSET)

        _role = d.pop("role", UNSET)
        role: Union[Unset, GmpTrackConfirmationDtoRole]
        if isinstance(_role, Unset):
            role = UNSET
        else:
            role = GmpTrackConfirmationDtoRole(_role)

        gmp_track_confirmation_dto = cls(
            user=user,
            when=when,
            approval=approval,
            confirmed_date=confirmed_date,
            comment=comment,
            role=role,
        )

        gmp_track_confirmation_dto.additional_properties = d
        return gmp_track_confirmation_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class OnlineEquipmentNodeVariableDtoEquipmentType(str, Enum):
    ACTOR = "ACTOR"
    METAINFO = "METAINFO"
    SENSOR = "SENSOR"
    SETPOINT = "SETPOINT"
    STATE = "STATE"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from qubicon.api.types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.xlsx_parsing_error_model_throwable_stack_trace_item_include_info_flags_parent_defined_packages_item import (
        XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParentDefinedPackagesItem,
    )
    from ..models.xlsx_parsing_error_model_throwable_stack_trace_item_include_info_flags_parent_unnamed_module import (
        XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParentUnnamedModule,
    )


T = TypeVar("T", bound="XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParent")


@_attrs_define
class XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParent:
    """
    Attributes:
        unnamed_module (Union[Unset, XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParentUnnamedModule]):
        default_assertion_status (Union[Unset, bool]):
        name (Union[Unset, str]):
        defined_packages (Union[Unset,
            List['XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParentDefinedPackagesItem']]):
        default_assertion_status_impl (Union[Unset, bool]):
        registered_as_parallel_capable (Union[Unset, bool]):
    """

    unnamed_module: Union[Unset, "XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParentUnnamedModule"] = (
        UNSET
    )
    default_assertion_status: Union[Unset, bool] = UNSET
    name: Union[Unset, str] = UNSET
    defined_packages: Union[
        Unset, List["XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParentDefinedPackagesItem"]
    ] = UNSET
    default_assertion_status_impl: Union[Unset, bool] = UNSET
    registered_as_parallel_capable: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        unnamed_module: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.unnamed_module, Unset):
            unnamed_module = self.unnamed_module.to_dict()

        default_assertion_status = self.default_assertion_status

        name = self.name

        defined_packages: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.defined_packages, Unset):
            defined_packages = []
            for defined_packages_item_data in self.defined_packages:
                defined_packages_item = defined_packages_item_data.to_dict()
                defined_packages.append(defined_packages_item)

        default_assertion_status_impl = self.default_assertion_status_impl

        registered_as_parallel_capable = self.registered_as_parallel_capable

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if unnamed_module is not UNSET:
            field_dict["unnamedModule"] = unnamed_module
        if default_assertion_status is not UNSET:
            field_dict["defaultAssertionStatus"] = default_assertion_status
        if name is not UNSET:
            field_dict["name"] = name
        if defined_packages is not UNSET:
            field_dict["definedPackages"] = defined_packages
        if default_assertion_status_impl is not UNSET:
            field_dict["defaultAssertionStatusImpl"] = default_assertion_status_impl
        if registered_as_parallel_capable is not UNSET:
            field_dict["registeredAsParallelCapable"] = registered_as_parallel_capable

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.xlsx_parsing_error_model_throwable_stack_trace_item_include_info_flags_parent_defined_packages_item import (
            XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParentDefinedPackagesItem,
        )
        from ..models.xlsx_parsing_error_model_throwable_stack_trace_item_include_info_flags_parent_unnamed_module import (
            XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParentUnnamedModule,
        )

        d = src_dict.copy()
        _unnamed_module = d.pop("unnamedModule", UNSET)
        unnamed_module: Union[Unset, XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParentUnnamedModule]
        if isinstance(_unnamed_module, Unset):
            unnamed_module = UNSET
        else:
            unnamed_module = XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParentUnnamedModule.from_dict(
                _unnamed_module
            )

        default_assertion_status = d.pop("defaultAssertionStatus", UNSET)

        name = d.pop("name", UNSET)

        defined_packages = []
        _defined_packages = d.pop("definedPackages", UNSET)
        for defined_packages_item_data in _defined_packages or []:
            defined_packages_item = (
                XlsxParsingErrorModelThrowableStackTraceItemIncludeInfoFlagsParentDefinedPackagesItem.from_dict(
                    defined_packages_item_data
                )
            )

            defined_packages.append(defined_packages_item)

        default_assertion_status_impl = d.pop("defaultAssertionStatusImpl", UNSET)

        registered_as_parallel_capable = d.pop("registeredAsParallelCapable", UNSET)

        xlsx_parsing_error_model_throwable_stack_trace_item_include_info_flags_parent = cls(
            unnamed_module=unnamed_module,
            default_assertion_status=default_assertion_status,
            name=name,
            defined_packages=defined_packages,
            default_assertion_status_impl=default_assertion_status_impl,
            registered_as_parallel_capable=registered_as_parallel_capable,
        )

        xlsx_parsing_error_model_throwable_stack_trace_item_include_info_flags_parent.additional_properties = d
        return xlsx_parsing_error_model_throwable_stack_trace_item_include_info_flags_parent

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from .. import errors
from ..client import AuthenticatedClient, Client
from ...models.user_login_dto import UserLoginDto
from qubicon.api.types import Response


def _get_kwargs(
    *,
    body: UserLoginDto,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {
        "Content-Type": "application/json",  # Explicitly set the Content-Type
    }

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/public-api/login",
        "json": body.to_dict(),  # Send the body as JSON
        "headers": headers,      # No Authorization header
    }

    return _kwargs



def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: UserLoginDto,
) -> Response[Any]:
    """
    Perform a login request with no additional headers or authorization.

    Args:
        client: The client to use for the request.
        body: The login data.

    Returns:
        Response[Any]: The response from the server.
    """
    # Extract the base_url from the client
    base_url = client.base_url if hasattr(client, "base_url") else getattr(client, "_base_url", None)
    if not base_url:
        raise ValueError("The 'base_url' is not set on the client object.")
    
    # Define headers explicitly to ensure no 'Authorization' or JWT headers are included
    headers = {
        "Content-Type": "application/json",  # Match Postman behavior
    }

    # Create a direct httpx client and make the POST request
    with httpx.Client(base_url=base_url, headers=headers) as http_client:
        response = http_client.post(
            "/public-api/login",
            json=body.to_dict(),  # Ensure the payload matches Postman
        )

    # Log the response for debugging purposes
    print("Request Headers:", headers)
    print("Request Payload:", body.to_dict())
    print("Response Status:", response.status_code)
    print("Response Body:", response.text)

    # Return the response as expected by the rest of the system
    return _build_response(client=client, response=response)



async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: UserLoginDto,
) -> Response[Any]:
    """
    Args:
        body (UserLoginDto):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from .. import errors
from ..client import AuthenticatedClient, Client
from ...models.refresh_token_dto import RefreshTokenDto
from qubicon.api.types import UNSET, Response, Unset


def _get_kwargs(
    *,
    body: RefreshTokenDto,
    normal: Union[Unset, bool] = UNSET,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    params: Dict[str, Any] = {}

    params["normal"] = normal

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "put",
        "url": "/public-api/refresh",
        "params": params,
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: RefreshTokenDto,
    normal: Union[Unset, bool] = UNSET,
) -> Response[Any]:
    """
    Args:
        normal (Union[Unset, bool]):
        body (RefreshTokenDto):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        body=body,
        normal=normal,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: RefreshTokenDto,
    normal: Union[Unset, bool] = UNSET,
) -> Response[Any]:
    """
    Args:
        normal (Union[Unset, bool]):
        body (RefreshTokenDto):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        body=body,
        normal=normal,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.minimum_process_group_dto_status import MinimumProcessGroupDtoStatus
from qubicon.api.types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.minimum_process_dto import MinimumProcessDto
    from ..models.online_equipment_dto import OnlineEquipmentDto


T = TypeVar("T", bound="MinimumProcessGroupDto")


@_attrs_define
class MinimumProcessGroupDto:
    """
    Attributes:
        name (str):
        id (Union[Unset, int]):
        creation_date (Union[Unset, int]):
        last_usage_date (Union[Unset, int]):
        part_of_experiment_equipment_group (Union[Unset, OnlineEquipmentDto]):
        update_date (Union[Unset, int]):
        manually_created (Union[Unset, bool]):
        has_charts (Union[Unset, bool]):
        status (Union[Unset, MinimumProcessGroupDtoStatus]):
        processes (Union[Unset, List['MinimumProcessDto']]):
    """

    name: str
    id: Union[Unset, int] = UNSET
    creation_date: Union[Unset, int] = UNSET
    last_usage_date: Union[Unset, int] = UNSET
    part_of_experiment_equipment_group: Union[Unset, "OnlineEquipmentDto"] = UNSET
    update_date: Union[Unset, int] = UNSET
    manually_created: Union[Unset, bool] = UNSET
    has_charts: Union[Unset, bool] = UNSET
    status: Union[Unset, MinimumProcessGroupDtoStatus] = UNSET
    processes: Union[Unset, List["MinimumProcessDto"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        id = self.id

        creation_date = self.creation_date

        last_usage_date = self.last_usage_date

        part_of_experiment_equipment_group: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.part_of_experiment_equipment_group, Unset):
            part_of_experiment_equipment_group = self.part_of_experiment_equipment_group.to_dict()

        update_date = self.update_date

        manually_created = self.manually_created

        has_charts = self.has_charts

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        processes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.processes, Unset):
            processes = []
            for processes_item_data in self.processes:
                processes_item = processes_item_data.to_dict()
                processes.append(processes_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if creation_date is not UNSET:
            field_dict["creationDate"] = creation_date
        if last_usage_date is not UNSET:
            field_dict["lastUsageDate"] = last_usage_date
        if part_of_experiment_equipment_group is not UNSET:
            field_dict["partOfExperimentEquipmentGroup"] = part_of_experiment_equipment_group
        if update_date is not UNSET:
            field_dict["updateDate"] = update_date
        if manually_created is not UNSET:
            field_dict["manuallyCreated"] = manually_created
        if has_charts is not UNSET:
            field_dict["hasCharts"] = has_charts
        if status is not UNSET:
            field_dict["status"] = status
        if processes is not UNSET:
            field_dict["processes"] = processes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.minimum_process_dto import MinimumProcessDto
        from ..models.online_equipment_dto import OnlineEquipmentDto

        d = src_dict.copy()
        name = d.pop("name")

        id = d.pop("id", UNSET)

        creation_date = d.pop("creationDate", UNSET)

        last_usage_date = d.pop("lastUsageDate", UNSET)

        _part_of_experiment_equipment_group = d.pop("partOfExperimentEquipmentGroup", UNSET)
        part_of_experiment_equipment_group: Union[Unset, OnlineEquipmentDto]
        if isinstance(_part_of_experiment_equipment_group, Unset):
            part_of_experiment_equipment_group = UNSET
        else:
            part_of_experiment_equipment_group = OnlineEquipmentDto.from_dict(_part_of_experiment_equipment_group)

        update_date = d.pop("updateDate", UNSET)

        manually_created = d.pop("manuallyCreated", UNSET)

        has_charts = d.pop("hasCharts", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, MinimumProcessGroupDtoStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = MinimumProcessGroupDtoStatus(_status)

        processes = []
        _processes = d.pop("processes", UNSET)
        for processes_item_data in _processes or []:
            processes_item = MinimumProcessDto.from_dict(processes_item_data)

            processes.append(processes_item)

        minimum_process_group_dto = cls(
            name=name,
            id=id,
            creation_date=creation_date,
            last_usage_date=last_usage_date,
            part_of_experiment_equipment_group=part_of_experiment_equipment_group,
            update_date=update_date,
            manually_created=manually_created,
            has_charts=has_charts,
            status=status,
            processes=processes,
        )

        minimum_process_group_dto.additional_properties = d
        return minimum_process_group_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.abstract_job_result_job_type import AbstractJobResultJobType
from qubicon.api.types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.abstract_import_file_progress import AbstractImportFileProgress


T = TypeVar("T", bound="JobSamplingOfflineDataImportResult")


@_attrs_define
class JobSamplingOfflineDataImportResult:
    """
    Attributes:
        job_type (Union[Unset, AbstractJobResultJobType]):
        result (Union[Unset, AbstractImportFileProgress]):
    """

    job_type: Union[Unset, AbstractJobResultJobType] = UNSET
    result: Union[Unset, "AbstractImportFileProgress"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        job_type: Union[Unset, str] = UNSET
        if not isinstance(self.job_type, Unset):
            job_type = self.job_type.value

        result: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.result, Unset):
            result = self.result.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if job_type is not UNSET:
            field_dict["jobType"] = job_type
        if result is not UNSET:
            field_dict["result"] = result

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.abstract_import_file_progress import AbstractImportFileProgress

        d = src_dict.copy()
        _job_type = d.pop("jobType", UNSET)
        job_type: Union[Unset, AbstractJobResultJobType]
        if isinstance(_job_type, Unset):
            job_type = UNSET
        else:
            job_type = AbstractJobResultJobType(_job_type)

        _result = d.pop("result", UNSET)
        result: Union[Unset, AbstractImportFileProgress]
        if isinstance(_result, Unset):
            result = UNSET
        else:
            result = AbstractImportFileProgress.from_dict(_result)

        job_sampling_offline_data_import_result = cls(
            job_type=job_type,
            result=result,
        )

        job_sampling_offline_data_import_result.additional_properties = d
        return job_sampling_offline_data_import_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from qubicon.api.types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.channel_data_pair_dto import ChannelDataPairDto


T = TypeVar("T", bound="MultiplexChartDataDto")


@_attrs_define
class MultiplexChartDataDto:
    """
    Attributes:
        channels (Union[Unset, List['ChannelDataPairDto']]):
    """

    channels: Union[Unset, List["ChannelDataPairDto"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        channels: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.channels, Unset):
            channels = []
            for channels_item_data in self.channels:
                channels_item = channels_item_data.to_dict()
                channels.append(channels_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if channels is not UNSET:
            field_dict["channels"] = channels

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.channel_data_pair_dto import ChannelDataPairDto

        d = src_dict.copy()
        channels = []
        _channels = d.pop("channels", UNSET)
        for channels_item_data in _channels or []:
            channels_item = ChannelDataPairDto.from_dict(channels_item_data)

            channels.append(channels_item)

        multiplex_chart_data_dto = cls(
            channels=channels,
        )

        multiplex_chart_data_dto.additional_properties = d
        return multiplex_chart_data_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ChannelDataKeyDtoType(str, Enum):
    OFFLINE = "OFFLINE"
    ONLINE = "ONLINE"
    SETPOINT = "SETPOINT"
    TAG = "TAG"

    def __str__(self) -> str:
        return str(self.value)
